/**
 * Infrared remote control signal Tx
 *  
 * MPLAB X IDE v5.10 with XC8 Ver2.05
 * (c)2020 Futoshi Sumikawa.
 */

#include "irtxsync.h"

void (*_irtxOnHandler)(void);
void (*_irtxOffHandler)(void);
void (*_irtxResetTimer)(void);
uint16_t (*_irtxReadTimer)(void);

#define _irtxHigh() _irtxOnHandler()
#define _irtxLow() _irtxOffHandler()

uint8_t* irtxData;

// Test 5key Panasonic LED ON
//uint8_t irFrameData[32] = {0x07, 0x40, 0x03, 0x83, 0x00, 0xFD, 0x00, 0x00,
//                           0x00, 0xCC, 0x02, 0x8B, 0x79, 0x00, 0x00, 0x28,
//                           0x34, 0x4A, 0x90, 0xB4, 0x24, 0x00, 0x00, 0x00,
//                           0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00};


// IR frame data format.
//
//  00  01  02  03  04  05  06  07  08  09  0A  0B  0C  0D  0E  0F
// [LeadH ][LeadL ][DataHS][DataHL][DataLS][DataLL][Trail ][Number]
//  10  11  12  13  14  15  16  17  18  19  1A  1B  1C  1D  1E  1F
// [D0][D1][D2][D3][D4][D5][D6][D7][D8][D9][DA][DB][DC][DD][DE][DF]
//
// LeadH: LeaderH x 2us
// LeadL: LeaderL x 2us
// DataHS: HShort x 2us
// DataHL: HLong  x 2us
// DataLS: LShort x 2us
// DataLL: LLong  x 2us
// Trail: Trailer x 2us
// Number : Number of data bits
// D0-DF: Data 128bit
//


// AEHA Format.
// |~~~~~~~~|____|~|_|~|___...|~|____________|~~~~...
//  Leader        0   1        Trailer
//  8T       4T   T T T 3T     (>=8ms)130ms->|Leader
// T = 350 - 500 (425us typ.)
// Variable length data (48bit typ.)


// NEC Format.
// |~~~~~~~~~~~~~~~~|________|~|_|~|___...|~|____________|~~~~...
//  Leader                    0   1        Trailer    
//  16T              8T       T T T 3T            108ms->|Leader
// T = 562us
// Data 32bit


// Sony Format.
// |~~~~~~|_|~|_|~~|...____________|~~~~...
//  Leader 0   1             45ms->|Leader
//  4T     T T T 2T
// T = 600us
// Data 12bit or 15bit or 20bit


// Timeout 100ms (2us x 50000)
#define IRRX_CYCLE_TIMEOUT 50000
// 8ms (2us x 4000)
#define IRRX_CYCLE_8ms 4000 

#define IRTX_INDEX_NONE 0
#define IRTX_INDEX_LEADER_H 1
#define IRTX_INDEX_LEADER_L 2
#define IRTX_INDEX_DATA_H 3
#define IRTX_INDEX_DATA_L 4
#define IRTX_INDEX_STOP_BIT 5
#define IRTX_INDEX_TRAILER 6

uint8_t _irtxFrameIndex = IRTX_INDEX_NONE;
uint8_t _irtxDataIndex = 0;
uint8_t _irtxDataValue = 0;

// ----
uint16_t _irtxDataLeaderH = 0;
uint16_t _irtxDataLeaderL = 0;

uint16_t _irtxDataHShort = 0;
uint16_t _irtxDataHLong = 0;
uint16_t _irtxDataLShort = 0;
uint16_t _irtxDataLLong = 0;

uint16_t _irtxDataNumber = 0;

uint16_t _irtxDataTrailer = 0;
// ----

uint16_t _irtxDataBit = 0;


uint8_t _irtxGetBitData(uint8_t index) {
    
    uint8_t dataNumber = index >> 3;
    uint8_t bitNumber = index - (dataNumber << 3);
    uint8_t target = irtxData[16 + dataNumber];
    
    if (((target << bitNumber) & 0x80) == 0) {
        return 0;
    }
    return 1;
}

uint16_t _irtxTimerOffset = 0;

void _irtxWaitTimer(uint16_t bounds) {
    _irtxTimerOffset = _irtxTimerOffset + bounds;
    while (_irtxReadTimer() < _irtxTimerOffset) {
        // Wait for bounds x 2us
    }
        while (_irtxReadTimer() < _irtxTimerOffset) {
        // Wait for bounds x 2us
    }
    
}

void _irtxSendFrame() {

    _irtxFrameIndex = IRTX_INDEX_LEADER_H;

    _irtxTimerOffset = 0;

    uint8_t GIEBitValue = INTCONbits.GIE;
    // Disable INTs
    INTCONbits.GIE = 0;

    _irtxResetTimer();
        
    // Leader
    _irtxHigh();
    _irtxFrameIndex = IRTX_INDEX_LEADER_H;
    _irtxWaitTimer(_irtxDataLeaderH);
    _irtxLow();
    _irtxFrameIndex = IRTX_INDEX_LEADER_L;
    _irtxWaitTimer(_irtxDataLeaderL);

    // Data bits
    _irtxHigh();
    _irtxFrameIndex = IRTX_INDEX_DATA_H;
    _irtxDataIndex = 0;
    _irtxDataValue = _irtxGetBitData(0);

    while (_irtxFrameIndex == IRTX_INDEX_DATA_H || _irtxFrameIndex == IRTX_INDEX_DATA_L) {
    
        if (_irtxFrameIndex == IRTX_INDEX_DATA_H) {

            if (_irtxDataValue) {
                _irtxDataBit = _irtxDataHLong;
            } else {
                _irtxDataBit = _irtxDataHShort;
            }
            _irtxWaitTimer(_irtxDataBit);
            _irtxLow();
            _irtxFrameIndex = IRTX_INDEX_DATA_L;
        } else {

            if (_irtxDataValue) {
                _irtxDataBit = _irtxDataLLong;
            } else {
                _irtxDataBit = _irtxDataLShort;
            }
            _irtxWaitTimer(_irtxDataBit);
            _irtxDataIndex++;
            if (_irtxDataIndex == _irtxDataNumber) {
                if (_irtxDataHLong == _irtxDataHShort) {
                    _irtxHigh();
                    _irtxFrameIndex = IRTX_INDEX_STOP_BIT;
                } else {
                    _irtxFrameIndex = IRTX_INDEX_TRAILER;
                }
            } else {
                _irtxHigh();
                _irtxFrameIndex = IRTX_INDEX_DATA_H;
                _irtxDataValue = _irtxGetBitData(_irtxDataIndex);
            }
        }
    }    
    
    // Stop bit
    if (_irtxFrameIndex == IRTX_INDEX_STOP_BIT) {
        _irtxWaitTimer(_irtxDataHShort);
        _irtxLow();
        _irtxFrameIndex = IRTX_INDEX_TRAILER;
    }
    
    // Trailer
    _irtxTimerOffset = 0;
    _irtxResetTimer();
    _irtxWaitTimer(_irtxDataTrailer);
    _irtxFrameIndex = IRTX_INDEX_NONE;

    INTCONbits.GIE = GIEBitValue;    
}

void irtxBegin(void (* onHandler)(void), void (* offHandler)(void), void (* resetTimer)(void), uint16_t (* readTimer)(void)) {
    _irtxOnHandler = onHandler;
    _irtxOffHandler = offHandler;
    _irtxResetTimer = resetTimer;
    _irtxReadTimer = readTimer;
}

void irtxSend(uint8_t* data) {
    irtxData = data;
    
    if (!(irtxData[0] == 0 && irtxData[1] == 0) && !(irtxData[0] == 0xff && irtxData[1] == 0xff)) {

        // Leader H (x2us)
        _irtxDataLeaderH = irtxData[0];
        _irtxDataLeaderH = (_irtxDataLeaderH << 8) + irtxData[1];
        // Leader L (x2us)
        _irtxDataLeaderL = irtxData[2];
        _irtxDataLeaderL = (_irtxDataLeaderL << 8) + irtxData[3];
        // Data H short (x2us)
        _irtxDataHShort = irtxData[4];
        _irtxDataHShort = (_irtxDataHShort << 8) + irtxData[5];
        // Data H long (x2us)
        _irtxDataHLong = irtxData[6];
        _irtxDataHLong = (_irtxDataHLong << 8) + irtxData[7];
        // Data L short (x2us)
        _irtxDataLShort = irtxData[8];
        _irtxDataLShort = (_irtxDataLShort << 8) + irtxData[9];
        // Data L long (x2us)
        _irtxDataLLong = irtxData[10];
        _irtxDataLLong = (_irtxDataLLong << 8) + irtxData[11];

        // Trailer (x2us)
        _irtxDataTrailer = irtxData[12];
        _irtxDataTrailer = (_irtxDataTrailer << 8) + irtxData[13];
        // Minimum value limit. (8ms)
        if (_irtxDataTrailer < IRRX_CYCLE_8ms) {
            _irtxDataTrailer = IRRX_CYCLE_8ms;
        }
        
        // Number
        // Maximum value limit. (128bit)
        _irtxDataNumber = irtxData[15] & 0b01111111;
        
      
  
        if (_irtxDataHLong == 0 || _irtxDataLLong == 0) {
            
            if (_irtxDataHLong == 0) {
                _irtxDataHLong = _irtxDataHShort;
            }         
            if (_irtxDataLLong == 0) {
                _irtxDataLLong = _irtxDataLShort;
            }   

            _irtxSendFrame();
        }
        
    }
}


/**
 End of File
*/