/**
 * Display type learning remote control
 * 
 *             1 VDD        20 VSS
 * IRinPower   2 RA5        19 RA0 SW4
 * IRin        3 RA4        18 RA1 SW6
 * SW12        4 RA3        17 RA2 IRout (PWM5)
 * SW11        5 RC5        16 RC0 SW2
 * SW10        6 RC4        15 RC1 SW1
 * SW9         7 RC3        14 RC2 LED
 * SW8         8 RC6        13 RB4 SW3 
 * SW7         9 RC7        12 RB5 SDA1
 * SCL1       10 RB7        11 RB6 SW5
 *  
 * PIC16F18346 MPLAB X IDE v5.10 with XC8 Ver2.05
 * (c)2020 Futoshi Sumikawa. 2020/03/02
 * 
 * PIC16F18346 MPLAB X IDE v6.20 with XC8 Ver2.46
 * (c)2024 Futoshi Sumikawa. 2024/07/07
 */

/**
  Generated Main Source File

  Company:
    Microchip Technology Inc.

  File Name:
    main.c

  Summary:
    This is the main file generated using PIC10 / PIC12 / PIC16 / PIC18 MCUs

  Description:
    This header file provides implementations for driver APIs for all modules selected in the GUI.
    Generation Information :
        Product Revision  :  PIC10 / PIC12 / PIC16 / PIC18 MCUs - 1.76
        Device            :  PIC16F18346
        Driver Version    :  2.00
*/

/*
    (c) 2018 Microchip Technology Inc. and its subsidiaries. 
    
    Subject to your compliance with these terms, you may use Microchip software and any 
    derivatives exclusively with Microchip products. It is your responsibility to comply with third party 
    license terms applicable to your use of third party software (including open source software) that 
    may accompany Microchip software.
    
    THIS SOFTWARE IS SUPPLIED BY MICROCHIP "AS IS". NO WARRANTIES, WHETHER 
    EXPRESS, IMPLIED OR STATUTORY, APPLY TO THIS SOFTWARE, INCLUDING ANY 
    IMPLIED WARRANTIES OF NON-INFRINGEMENT, MERCHANTABILITY, AND FITNESS 
    FOR A PARTICULAR PURPOSE.
    
    IN NO EVENT WILL MICROCHIP BE LIABLE FOR ANY INDIRECT, SPECIAL, PUNITIVE, 
    INCIDENTAL OR CONSEQUENTIAL LOSS, DAMAGE, COST OR EXPENSE OF ANY KIND 
    WHATSOEVER RELATED TO THE SOFTWARE, HOWEVER CAUSED, EVEN IF MICROCHIP 
    HAS BEEN ADVISED OF THE POSSIBILITY OR THE DAMAGES ARE FORESEEABLE. TO 
    THE FULLEST EXTENT ALLOWED BY LAW, MICROCHIP'S TOTAL LIABILITY ON ALL 
    CLAIMS IN ANY WAY RELATED TO THIS SOFTWARE WILL NOT EXCEED THE AMOUNT 
    OF FEES, IF ANY, THAT YOU HAVE PAID DIRECTLY TO MICROCHIP FOR THIS 
    SOFTWARE.
*/

#include "mcc_generated_files/mcc.h"
#include "irtxsync.h"
#include "irrxsync.h"
#include "st7032.h"

//#define ICSP_MODE

#define LCD_CONTRAST 50

// 3 min (200 ms x 900)
#define TIMEOUT_LEARN_MODE 900

// 7 sec (100 ms x 70)
#define TIMEOUT_SLEEP_MODE 70

#define DATAEE_ADDRESS 0x7000
#define FLASH_ADDRESS 0x3000
#define FLASH_PAGE_SIZE 0x0400

#define serialPutCR() EUSART_Write(0x0d)
#define serialPutLF() EUSART_Write(0x0a)

#define SW_ON 0
#define SW_OFF 1


#define LED_OFF 0
#define LED_ON 1
#define LED_BLINK 2


const uint8_t CHAR_SONY_00[8] = {
  0b10010,
  0b10010,
  0b10010,
  0b10010,
  0b10010,
  0b10010,
  0b10010,
  0b10010
};

const uint8_t CHAR_SONY_01[8] = {
  0b10011,
  0b10011,
  0b10011,
  0b10011,
  0b10011,
  0b10011,
  0b10011,
  0b10011
};

const uint8_t CHAR_SONY_10[8] = {
  0b11010,
  0b11010,
  0b11010,
  0b11010,
  0b11010,
  0b11010,
  0b11010,
  0b11010
};

const uint8_t CHAR_SONY_11[8] = {
  0b11011,
  0b11011,
  0b11011,
  0b11011,
  0b11011,
  0b11011,
  0b11011,
  0b11011
};

const uint8_t CHAR_SONY_1_[8] = {
  0b11000,
  0b11000,
  0b11000,
  0b11000,
  0b11000,
  0b11000,
  0b11000,
  0b11000
};

const uint8_t CHAR_SONY_0_[8] = {
  0b10000,
  0b10000,
  0b10000,
  0b10000,
  0b10000,
  0b10000,
  0b10000,
  0b10000
};

const uint8_t CHAR_AEHA_A[8] = {
  0b10101,
  0b10101,
  0b10101,
  0b10101,
  0b10101,
  0b10101,
  0b10101,
  0b10101
};

const uint8_t CHAR_AEHA_B[8] = {
  0b10001,
  0b10001,
  0b10001,
  0b10001,
  0b10001,
  0b10001,
  0b10001,
  0b10001
};

const uint8_t CHAR_AEHA_C[8] = {
  0b10100,
  0b10100,
  0b10100,
  0b10100,
  0b10100,
  0b10100,
  0b10100,
  0b10100
};

const uint8_t CHAR_AEHA_D[8] = {
  0b00101,
  0b00101,
  0b00101,
  0b00101,
  0b00101,
  0b00101,
  0b00101,
  0b00101
};

const uint8_t CHAR_AEHA_E[8] = {
  0b00100,
  0b00100,
  0b00100,
  0b00100,
  0b00100,
  0b00100,
  0b00100,
  0b00100
};

const uint8_t CHAR_AEHA_F[8] = {
  0b10000,
  0b10000,
  0b10000,
  0b10000,
  0b10000,
  0b10000,
  0b10000,
  0b10000
};


uint8_t learnMode = 0;
uint16_t timerLearnMode = 0;
uint16_t timerSleepMode = 0;

uint8_t dataPage = 0;
uint16_t blockData[32];

uint8_t isWakeup = 0;

// Test 5key Panasonic LED ON
uint8_t irFrameData[32] = {0x07, 0x40, 0x03, 0x83, 0x00, 0xfd, 0x00, 0x00, 0x00, 0xcc, 0x02, 0x8b, 0x79, 0x00, 0x00, 0x28, 0x34, 0x4a, 0x90, 0xb4, 0x24, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00};

uint8_t binToHex(uint8_t data) {

    uint8_t v = data & 0x0f;
    if (v < 10) {
        v = '0' + v;
    } else {
        v = 'A' + v - 10;
    }
    return v;
}

void lcdPutHexData(uint8_t data) {
    
    uint8_t h = binToHex(data >> 4);
    lcdPutc(h);
    uint8_t l = binToHex(data);
    lcdPutc(l);
}

void lcdPutDecData(uint8_t data) {
    
    uint8_t t = data / 10;
    lcdPutc('0' + t);
    uint8_t o = data % 10;
    lcdPutc('0' + o);
}

uint8_t lcdGetFrameBit(uint8_t number) {
    
    uint8_t byte = number / 8;
    uint8_t bit = number % 8;
    
    uint8_t value = irFrameData[16 + byte];
    return (value >> (7 - bit)) & 0b00000001;                
}



void lcdDisplayingFrameDataInGexadecimal(uint8_t bit, uint8_t index, uint8_t byte) {

    uint8_t ix = 0;

    for (uint8_t i = index; i < index + byte; i++) {
        uint8_t v = irFrameData[i];
        if (ix < bit) {
            uint8_t r = bit - ix;
            if (r < 8) {
                uint8_t s = 8 - r;
                v = v >> s;
                v = v << s;
                if (r <= 4) {
                    lcdPutc(binToHex(v >> 4));
                    lcdPutc(' ');
                } else {
                    lcdPutHexData(v);
                }
            } else {
                lcdPutHexData(v);
            }
        } else {
            lcdPutc(' ');
            lcdPutc(' ');
        }
        ix += 8;
    }
}


void lcdPutFrameData() {

    // LCD power on
    IRinPower_SetLow();
    
    uint16_t leader = irFrameData[0];
    leader = (leader << 8) + irFrameData[1];

    uint16_t dataHighShort = irFrameData[4];
    dataHighShort = (dataHighShort << 8) + irFrameData[5];

    uint16_t dataHighLong = irFrameData[6];
    dataHighLong = (dataHighLong << 8) + irFrameData[7];
    
    lcdSetCursor(0,0);
    
    if (leader == 0xffff || leader == 0) {
        // No data to send Let's register  
        lcdPuts("No data to send ");
        lcdSetCursor(0,1);
        lcdPuts("Let's register  ");
        return;
    }

    // Number of data bits
    uint8_t n = irFrameData[15];
        
    // Type
    uint8_t type = 0;
    if (dataHighLong > 0) {
        // SONY
        type = 'S';

        lcdPuts("SONY ");

        // Data
        uint8_t data0 = irFrameData[16];
        uint8_t data1 = irFrameData[17];
        uint8_t data2 = irFrameData[18];

        // Data
        uint8_t data = data0 >> 1;
        lcdPutHexData(data);
        lcdPutc(' ');

        // Address
        if (n == 20) {
            // 13-bit address (8-bit + 5-bit))
            uint8_t address0 = ((data0 << 4) & 0x00010000) | (data1 >> 4);
            lcdPutHexData(address0);
            uint8_t address1 = ((data1 << 4) | (data2 >> 4));
            lcdPutHexData(address1);
            lcdPuts("    ");
            
        } else if (n == 15) {
            // 8-bit address
            uint8_t address0 = (data0 << 7) | (data1 >> 1);
            lcdPutHexData(address0);
            lcdPuts("(8bit)");

        } else {
            // 5-bit address
            uint8_t address0 = ((data0 << 4) & 0x00010000) | (data1 >> 4);
            lcdPutHexData(address0);            
            lcdPuts("(5bit)");

        }

    } else {

        // Customer Code
        uint8_t codeH = irFrameData[16];
        uint8_t codeL = irFrameData[17];
        
        // Data
        uint8_t data0 = irFrameData[18];
        uint8_t data1 = irFrameData[19];

        uint8_t parity = codeH >> 4;
        parity = parity ^ (codeH & 0xf);
        parity = parity ^ codeL >> 4;
        parity = parity ^ (codeL & 0xf);

        if (n == 32 & (data0 == (data1 ^ 0xff))) {
            // 8-bit data + 8-bit inverted data.
            // NEC Format.
            type = 'N';

            lcdPuts("NEC ");
            // Customer Code
            lcdPutHexData(codeH);
            lcdPutHexData(codeL);
            lcdPutc(' ');
            // Data
            lcdPutHexData(irFrameData[18]);
            lcdPuts("     ");

        } else if (parity == (data0 >> 4)) {
            // Parity check.
            // AEHA Format.
            type = 'A';

            if (n == 48) {
                // Data: 28-bit
                lcdPuts("AEHA ");
            } else {
                lcdPuts("AE");
                lcdPutDecData(n - 20);
                lcdPutc(' ');
            }            
            
            // Customer Code
            lcdPutHexData(codeH);
            lcdPutHexData(codeL);
            // Data0
            lcdPutc(binToHex(data0 >> 4));            
            // Data1 - DataN
            lcdDisplayingFrameDataInGexadecimal(n - 24, 19, 3);
          
        } else {
            // Other formats.
            type = 'O';
            
            lcdPutDecData(n);
            lcdPuts("bit ");

            // Data
            lcdDisplayingFrameDataInGexadecimal(n, 16, 5);

        }
    }
    
    // View the graph.
    
    if (type == 'S') {
        
        // SONY format.

        lcdRegisterChar(0, (uint8_t*)CHAR_SONY_00);
        lcdRegisterChar(1, (uint8_t*)CHAR_SONY_01);
        lcdRegisterChar(2, (uint8_t*)CHAR_SONY_10);
        lcdRegisterChar(3, (uint8_t*)CHAR_SONY_11);
        lcdRegisterChar(4, (uint8_t*)CHAR_SONY_0_);
        lcdRegisterChar(5, (uint8_t*)CHAR_SONY_1_);

        lcdSetCursor(0,1);

        uint8_t ix = 0;

        for (uint8_t i = 16; i < 20; i++) {

            uint8_t d = irFrameData[i];
            uint8_t b8 = d >> 6;
            uint8_t b6 = (d >> 4) & 0b00000011;
            uint8_t b4 = (d >> 2) & 0b00000011;
            uint8_t b2 = d & 0b00000011;

            if (ix < n) {
                if (ix + 1 == n) {
                    b8 = b8 >> 1;
                    lcdPutc(4 + b8);
                } else {
                    lcdPutc(b8);
                }
                ix += 2;
            } else {
                lcdPutc(' ');
            }

            if (ix < n) {
                if (ix + 1 == n) {
                    b6 = b6 >> 1;
                    lcdPutc(4 + b6);
                } else {
                    lcdPutc(b6);
                }
                ix += 2;
            } else {
                lcdPutc(' ');
            }

            if (ix < n) {
                if (ix + 1 == n) {
                    b4 = b4 >> 1;
                    lcdPutc(4 + b4);
                } else {
                    lcdPutc(b4);
                }
                ix += 2;
            } else {
                lcdPutc(' ');
            }

            if (ix < n) {
                if (ix + 1 == n) {
                    b2 = b2 >> 1;
                    lcdPutc(4 + b2);
                } else {
                    lcdPutc(b2);
                }
                ix += 2;
            } else {
                lcdPutc(' ');
            }
        }

    } else {

        // AEHA, NEC, other formats.
        
        lcdRegisterChar(0, (uint8_t*)CHAR_AEHA_A);
        lcdRegisterChar(1, (uint8_t*)CHAR_AEHA_B);
        lcdRegisterChar(2, (uint8_t*)CHAR_AEHA_C);
        lcdRegisterChar(3, (uint8_t*)CHAR_AEHA_D);
        lcdRegisterChar(4, (uint8_t*)CHAR_AEHA_E);
        lcdRegisterChar(5, (uint8_t*)CHAR_AEHA_F);
        
        lcdSetCursor(0,1);
        
        uint8_t ix = 0;
        uint8_t font = 0;
        uint8_t decision = 1;
        uint8_t finish = 0;
        uint8_t bit = 0;
        uint8_t index = 0;
        
        while (finish == 0) {

            bit = lcdGetFrameBit(ix++);
            if (ix > n) {
                finish = 1;
            } else {
                // select char
                if (decision == 1) {
                    // first char (a or b or c)
                    if (bit == 0) {
                        // 0 (a or c)
                        bit = lcdGetFrameBit(ix++);
                        if (ix > n) {
                            finish = 1;
                        } else {

                            if (bit == 0) {
                                // 00 (a)
                                font = 'a';
                                bit = lcdGetFrameBit(ix++);
                                if (ix > n) {
                                    finish = 1;
                                } else {

                                    if (bit == 0) {
                                        // 000
                                        decision = 1;
                                    } else {
                                        // 001
                                        decision = 0;
                                    }
                                }
                            } else {
                                // 01 (c)
                                font = 'c';
                                decision = 1;
                            }
                        }
                        
                    } else {
                        // 1 (b)
                        font = 'b';
                        bit = lcdGetFrameBit(ix++);
                        if (ix > n) {
                            finish = 1;
                        } else {
                            if (bit == 0) {
                                // 10
                                decision = 1;
                            } else {
                                // 11
                                decision = 0;
                            }
                        }
                    }

                } else {
                    // second char (d or e)

                    if (bit == 0) {
                        // _0 (d)
                        font = 'd';
                        bit = lcdGetFrameBit(ix++);
                        if (ix > n) {
                            finish = 1;
                        } else {
                            if (bit == 0) {
                                // _00 (d)
                                decision = 1;
                            } else {
                                // _01 (d)
                                decision = 0;
                            }
                        }
                    } else {
                        // _1 (e)
                        font = 'e';
                        decision = 1;
                    }
                }
    
                // put char
                lcdPutc(font - 'a');
                index++;
            }
        }
        
        // stop bit
        if (decision == 1) {
            // first char
            font = 'f';
        } else {
            // second char
            font = 'e';
        }
        lcdPutc(font - 'a');
        index++;

        for (uint8_t i = 0; i < (16 - index); i++ ) {
            lcdPutc(' ');
        }
        
    }

 }

uint16_t getFlashAddress(uint8_t index) {
    uint16_t address = index;
    address = FLASH_ADDRESS + (address << 5) + (FLASH_PAGE_SIZE << dataPage);
    return address;
}

uint8_t compareFrameCodeData(uint8_t index) {

    uint16_t address = getFlashAddress(index);
    
    for (uint8_t i = 0; i < 16; i++) {
        if (irFrameData[16 + i] != DATAEE_ReadByte(address + 16 + i)) {
            return 0;
        }
    }
    return 1;
}

void saveFrameData(uint8_t index) {

    uint16_t address = getFlashAddress(index);

     for (uint8_t i = 0; i < 32; i++) {
        blockData[i] = irFrameData[i];
    }

    // write to Flash memory block
    FLASH_WriteBlock((uint16_t)address, (uint16_t*)blockData);

}

void loadFrameData(uint8_t index) {

    uint16_t address = getFlashAddress(index);
    
    for (uint8_t i = 0; i < 32; i++) {
        blockData[i] = FLASH_ReadWord(address + i);
    }
    
    for (uint8_t i = 0; i < 32; i++) {
        irFrameData[i] = blockData[i];
    }
}

void sendFrameData(uint8_t index) {
    loadFrameData(index);
    irtxSend(irFrameData);
}

uint8_t callbackIrInGetValue() {
    return IRin_GetValue();
}


void callbackIrtxOn() {
    // TODO Test
    LED_SetHigh();

    PWM5_LoadDutyValue(PWM5_INITIALIZE_DUTY_VALUE);
}

void callbackIrtxOff() {
    // TODO Test
    LED_SetLow();
    
    PWM5_LoadDutyValue(0);
}

void callbackResetTimer() {
    TMR1_WriteTimer(0);
}

uint16_t readTimer(void) {
    uint16_t readVal;
    uint8_t readValHigh;
    uint8_t readValLow;
	
    uint8_t preReadValLow = TMR1L;
    readValHigh = TMR1H;
    readValLow = TMR1L;
    if (readValLow < preReadValLow) {
        readValHigh = TMR1H;
    }
    readVal = ((uint16_t)readValHigh << 8) | readValLow;
    return readVal;
}


uint16_t callbackReadTimer() {
    return readTimer();
}

void deleteFrameData(uint8_t index) {
    irFrameData[0] = 0;
    irFrameData[1] = 0;
    saveFrameData(index);
}

void exitLearningMode() {
   learnMode = 0;
   IRinPower_SetHigh();
   LED_SetLow();   
}


/*
                         Main application
 */
void main(void)
{
    // initialize the device
    SYSTEM_Initialize();
    PWM5_LoadDutyValue(0);
    
    // When using interrupts, you need to set the Global and Peripheral Interrupt Enable bits
    // Use the following macros to:

    // Enable the Global Interrupts
    INTERRUPT_GlobalInterruptEnable();

    // Enable the Peripheral Interrupts
    INTERRUPT_PeripheralInterruptEnable();

    // Disable the Global Interrupts
    //INTERRUPT_GlobalInterruptDisable();

    // Disable the Peripheral Interrupts
    //INTERRUPT_PeripheralInterruptDisable();

    PWM5_LoadDutyValue(0);
    
    irtxBegin(callbackIrtxOn, callbackIrtxOff, callbackResetTimer, callbackReadTimer);
    irrxBegin(irFrameData, callbackIrInGetValue, callbackResetTimer, callbackReadTimer);
    
    uint8_t targetKey = 0;

    __delay_ms(300);

    uint8_t enableSW10 = 1;
    if (SW10_GetValue() == SW_ON) {
        enableSW10 = 0;
    }
    
    uint8_t cycleNumber200ms = 0;
    
    lcdBegin();
    lcdSetContrast(LCD_CONTRAST);
    
    lcdSetCursor(0,0);
    lcdPuts("Ir Learning");
    lcdSetCursor(0,1);
    lcdPuts("Remote Control");

    uint8_t currentKey = 0;
    
    while (1)
    {
        // Add your application code
        
        // Page
        dataPage = 0;
        
        // Switch
        uint8_t key = 0;
        
        if (SW1_GetValue() == SW_ON) {
            key = 1;
        } else if (SW2_GetValue() == SW_ON) {
            key = 2;
        } else if (SW3_GetValue() == SW_ON) {
            key = 3;
#ifndef ICSP_MODE
        } else if (SW4_GetValue() == SW_ON) {
            key = 4;
#endif
        } else if (SW5_GetValue() == SW_ON) {
            key = 5;
#ifndef ICSP_MODE
        } else if (SW6_GetValue() == SW_ON) {
            key = 6;
#endif
        } else if (SW7_GetValue() == SW_ON) {
            key = 7;
        } else if (SW8_GetValue() == SW_ON) {
            key = 8;
        } else if (SW9_GetValue() == SW_ON) {
            key = 9;
        } else if (SW11_GetValue() == SW_ON) {
            key = 11;
        } else if (SW12_GetValue() == SW_ON) {
            key = 12;
        } else if (SW10_GetValue() == SW_ON) {
            // MCLR
            if (enableSW10) {
                key = 10;
            }
        }
        
        // Mode
        if (SW7_GetValue() == SW_ON && SW8_GetValue() == SW_ON) {

            if (learnMode == 0) {
                // Learn mode.
                learnMode = 1;
                targetKey = 0;
                IRinPower_SetLow();
                LED_SetHigh();
                
                // Learning Choose a button
                lcdSetCursor(0,0);
                lcdPuts("Learning        ");
                lcdSetCursor(0,1);
                lcdPuts("Choose a button ");
                
                timerLearnMode = 0;
            } else {
                exitLearningMode();
                
                // Clesr
                lcdClear();

            }
            __delay_ms(50);
            while (SW7_GetValue() == SW_ON || SW8_GetValue() == SW_ON) {
                // Wait for key up.
            }
            __delay_ms(50);
            continue;
        }
        
        if (learnMode == 0) {
            // Send mode.
            if (key != 0) {
                
                if (currentKey != key) {
                    currentKey = key;
                    // Display frame data.
                    loadFrameData(key);
                    lcdPutFrameData();
                }
                
                sendFrameData(key);
                
                timerSleepMode = 0;
                
            } else {
                // Sleep
                if (SW1_GetValue() == SW_OFF
                        && SW2_GetValue() == SW_OFF 
                        && SW3_GetValue() == SW_OFF
#ifndef ICSP_MODE
                        && SW4_GetValue() == SW_OFF
#endif
                        && SW5_GetValue() == SW_OFF
#ifndef ICSP_MODE
                        && SW6_GetValue() == SW_OFF
#endif
                        && SW7_GetValue() == SW_OFF
                        && SW8_GetValue() == SW_OFF
                        && SW9_GetValue() == SW_OFF
                        && SW10_GetValue() == SW_OFF
                        && SW11_GetValue() == SW_OFF
                        && SW12_GetValue() == SW_OFF) {
                                        
                    if (readTimer() > 50000) {
                        TMR1_WriteTimer(0);
                        // 100ms cycle.

                        if (isWakeup == 1 && timerSleepMode == 3) {
                            isWakeup = 0;
                            lcdInitialize(LCD_CONTRAST);
                            lcdPutFrameData();
                        }

                        if (timerSleepMode++ > TIMEOUT_SLEEP_MODE) {
                            exitLearningMode();

                            // LCD power off
                            IRinPower_SetHigh();

//                            // Clear the LAT
//                            LATA = 0x00;
//                            LATB = 0x00;
//                            LATC = 0x00;
//
//                            // Switch TRIS to input
//                            TRISA = 0xff;
//                            TRISB = 0xff;
//                            TRISC = 0xff;

                            // Disable pullups on I2C
                            WPUB = 0b01010000;

                            // MSSP1MD: Disable MSSP1 bit
                            // 1 = MSSP1 module disabled
                            // 0 = MSSP1 module enable                                
                            MSSP1MD = 1;

                            SLEEP();
                            NOP();

                            // Enable pullups for I2C
                            WPUB = 0xF0;
//
//                            // Reverting TRIS
//                            TRISA = 0x13;
//                            TRISB = 0xF0;
//                            TRISC = 0xFB;

                            // MSSP1MD: Disable MSSP1 bit
                            // 1 = MSSP1 module disabled
                            // 0 = MSSP1 module enable                                
                            MSSP1MD = 0;

                            timerSleepMode = 0;

                            currentKey = 0;

                            isWakeup = 1;                                
                        }
                    }
                }
            }
        } else if (learnMode == 1) {
            // Learn mode.

            if (isWakeup == 1) {
                isWakeup = 0;
                lcdInitialize(LCD_CONTRAST);

                // Learning Choose a button
                lcdSetCursor(0,0);
                lcdPuts("Learning        ");
                lcdSetCursor(0,1);
                lcdPuts("Choose a button ");
            }
            
            // Learn
            uint8_t blink = 0;

            if (targetKey == 0) {
                
                if (key != 0) {
                    targetKey = key;
                }
                
            } else {

                // Waiting for signal reception
                lcdSetCursor(0,0);
                lcdPuts("Waiting for     ");
                lcdSetCursor(0,1);
                lcdPuts("signal reception");

                if (irrxIsReceiveComplete() == 1) {
                    // Received data.
                    LED_SetHigh();

                    // Save frame data.
                    saveFrameData(targetKey);
                    __delay_ms(300);

                    // Put frame data.
                    lcdPutFrameData();

                    targetKey = 0;
                } else {
                    // Timeout.
                }

                // Blink LED
                blink = 1;
            }

            if (readTimer() > 50000) {
                TMR1_WriteTimer(0);
                // 100ms cycle.
                if (cycleNumber200ms) {
                    cycleNumber200ms = 0;
                    // 200ms cycle.

                    if (blink) {
                        if (LED_GetValue() == 1) {
                            LED_SetLow();
                        } else {
                            LED_SetHigh();
                        }
                    }

                    if (timerLearnMode++ > TIMEOUT_LEARN_MODE) {
                        exitLearningMode();
                    }

                } else {
                    cycleNumber200ms = 1;
                }
            }
        }
    }
}
/**
 End of File
*/